// Copyright 2020 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

//#define LOG_NDEBUG 0
#define LOG_TAG "V4L2ComponentCommon"

#include <v4l2_codec2/v4l2/V4L2ComponentCommon.h>

#include <log/log.h>
#include <set>

namespace android {

const std::string V4L2ComponentName::kH264Encoder = "c2.v4l2.avc.encoder";

const std::string V4L2ComponentName::kH264Decoder = "c2.v4l2.avc.decoder";
const std::string V4L2ComponentName::kHEVCDecoder = "c2.v4l2.hevc.decoder";

// static
bool V4L2ComponentName::isValid(const std::string& name) {
    return name == kH264Encoder ||
           name == kH264Decoder ||
           name == kHEVCDecoder ;
}

// static
bool V4L2ComponentName::isEncoder(const std::string& name) {
    ALOG_ASSERT(isValid(name));

    return name == kH264Encoder;
}

// static
bool V4L2ComponentName::isDecoder(const std::string& name) {
    ALOG_ASSERT(isValid(name));
    static const std::set<std::string> kValidDecoders = {
            kH264Decoder,  kHEVCDecoder,
    };

    return kValidDecoders.find(name) != kValidDecoders.end();
}

// static
std::optional<VideoCodec> V4L2ComponentName::getCodec(const std::string& name) {
    ALOG_ASSERT(isValid(name));
    static const std::map<std::string, VideoCodec> kNameToCodecs = {
            {kH264Decoder, VideoCodec::H264},
            {kH264Encoder, VideoCodec::H264},

            {kHEVCDecoder, VideoCodec::HEVC},
    };

    auto iter = kNameToCodecs.find(name);
    if (iter == kNameToCodecs.end()) {
        return std::nullopt;
    }
    return iter->second;
}

}  // namespace android
